@@ -12,7 +12,6 @@ module Distribution.Solver.Modular.Preference
     , preferLinked
     , preferPackagePreferences
     , preferReallyEasyGoalChoices
-    , requireInstalled
     , onlyConstrained
     , sortGoals
     , pruneAfterFirstSuccess
@@ -86,18 +85,31 @@ preferLinked = addWeight (const (const linked))
 preferPackagePreferences :: (PN -> PackagePreferences) -> EndoTreeTrav d c
 preferPackagePreferences pcs =
     preferPackageStanzaPreferences pcs .
+    -- Each package is assigned a list of weights (currently three of them),
+    -- and options are ordered by comparison of these lists.
+    --
+    -- The head of the list (and thus the top priority for ordering)
+    -- is whether the package version is "preferred"
+    -- (https://hackage.haskell.org/packages/preferred-versions).
+    --
+    -- The next two elements depend on 'PackagePreferences'.
+    -- For 'PreferInstalled' they are whether the version is installed (0 or 1)
+    -- and how close is the version to the latest one (between 0.0 and 1.0).
+    -- For 'PreferLatest' the weights are the same, but swapped, so that
+    -- ordering considers how new is the package first.
+    -- For 'PreferOldest' one weight measures how close is the version to the
+    -- the oldest one possible (between 0.0 and 1.0) and another checks whether
+    -- the version is installed (0 or 1).
     addWeights [
           \pn _  opt -> preferred pn opt
-
-        -- Note that we always rank installed before uninstalled, and later
-        -- versions before earlier, but we can change the priority of the
-        -- two orderings.
         , \pn vs opt -> case preference pn of
                           PreferInstalled -> installed opt
                           PreferLatest    -> latest vs opt
+                          PreferOldest    -> oldest vs opt
         , \pn vs opt -> case preference pn of
                           PreferInstalled -> latest vs opt
                           PreferLatest    -> installed opt
+                          PreferOldest    -> installed opt
         ]
   where
     -- Prefer packages with higher version numbers over packages with
@@ -108,6 +120,11 @@ preferPackagePreferences pcs =
           index = fromMaybe l $ L.findIndex (<= version opt) sortedVersions
       in  fromIntegral index / fromIntegral l
 
+    -- Prefer packages with lower version numbers over packages with
+    -- higher version numbers.
+    oldest :: [Ver] -> POption -> Weight
+    oldest sortedVersions opt = 1 - latest sortedVersions opt
+
     preference :: PN -> InstalledPreference
     preference pn =
       let PackagePreferences _ ipref _ = pcs pn
@@ -300,18 +317,6 @@ enforceManualFlags pcs = go
       in W.mapWithKey (restrictToggling d flagConstraintValues) ts
     go x                                                            = x
 
--- | Require installed packages.
-requireInstalled :: (PN -> Bool) -> EndoTreeTrav d c
-requireInstalled p = go
-  where
-    go (PChoiceF v@(Q _ pn) rdm gr cs)
-      | p pn      = PChoiceF v rdm gr (W.mapWithKey installed cs)
-      | otherwise = PChoiceF v rdm gr                         cs
-      where
-        installed (POption (I _ (Inst _)) _) x = x
-        installed _ _ = Fail (varToConflictSet (P v)) CannotInstall
-    go x          = x
-
 -- | Avoid reinstalls.
 --
 -- This is a tricky strategy. If a package version is installed already and the
